import { visualizationColors } from 'config';
import React, { useRef, useEffect, useState } from 'react';
import Plot from 'react-plotly.js';
import { Tooltip, TooltipContent, TooltipTrigger } from '@graphpolaris/shared/lib/components/tooltip';
import { PlotType } from 'plotly.js';
import { select } from 'd3';

const getCSSVariableHSL = (varName: string) => {
  const rootStyles = getComputedStyle(document.documentElement);
  const hslValue = rootStyles.getPropertyValue(varName).trim().replace('deg', '');
  return `hsl(${hslValue})`;
};
export const plotTypeOptions = ['bar', 'scatter', 'line', 'histogram', 'pie'] as const;
export type SupportedPlotType = (typeof plotTypeOptions)[number];

export interface CustomChartPlotlyProps {
  xAxisData: string[] | number[];
  plotType: SupportedPlotType;
  title: string;
  showAxis: boolean;
  yAxisData: string[] | number[];
  xAxisLabel?: string;
  yAxisLabel?: string;
}

export const preparePlotData = (
  xAxisData: (string | number)[],
  plotType: SupportedPlotType,
  yAxisData?: (string | number)[],
  xAxisLabel?: string,
  yAxisLabel?: string,
  showAxis = true,
): { plotData: Partial<Plotly.PlotData>[]; layout: Partial<Plotly.Layout> } => {
  const primaryColor = getCSSVariableHSL('--clr-sec--400');
  const mainColors = visualizationColors.GPCat.colors[14];
  let xValues: (string | number)[] = [];
  let yValues: (string | number)[] = [];

  if (plotType === 'scatter' || plotType === 'line') {
    if (xAxisData.length !== 0 && yAxisData && yAxisData.length !== 0) {
      xValues = xAxisData;
      yValues = yAxisData;
    } else if (xAxisData.length !== 0 && (!yAxisData || yAxisData.length === 0)) {
      xValues = xAxisData;
      yValues = xAxisData.map((_, index) => index + 1);
    } else if (xAxisData.length === 0 && yAxisData && yAxisData.length !== 0) {
      xValues = yAxisData.map((_, index) => index + 1);
      yValues = yAxisData;
    }
  } else {
    xValues = xAxisData;
    yValues = xAxisData.map((_, index) => index + 1);
  }

  let sortedLabels: string[] = [];
  let sortedFrequencies = [];

  const plotData = (() => {
    switch (plotType) {
      case 'bar':
        return [
          {
            type: 'bar' as PlotType,
            x: xValues,
            y: yValues,
            marker: {
              color: primaryColor,
              line: {
                color: 'primaryColor',
                width: 1,
              },
            },
            customdata: xValues.map((label) => (label === 'undefined' || label === 'null' || label === '' ? 'nonData' : '')),
          },
        ];
      case 'scatter':
        return [
          {
            type: 'scatter' as PlotType,
            x: xValues,
            y: yValues,
            mode: 'markers' as 'markers',
            marker: { color: primaryColor, size: 12 },
            customdata: xValues.map((label) => (label === 'undefined' || label === 'null' || label === '' ? 'nonData' : '')),
          },
        ];
      case 'line':
        return [
          {
            type: 'scatter' as PlotType,
            x: xValues,
            y: yValues,
            mode: 'lines' as 'lines',
            line: { color: primaryColor },
            customdata: xValues.map((label) => (label === 'undefined' || label === 'null' || label === '' ? 'nonData' : '')),
          },
        ];
      case 'histogram':
        if (typeof xAxisData[0] === 'string') {
          const frequencyMap = xAxisData.reduce(
            (acc, item) => {
              acc[item] = (acc[item] || 0) + 1;
              return acc;
            },
            {} as Record<string, number>,
          );

          const sortedEntries = Object.entries(frequencyMap).sort((a, b) => b[1] - a[1]);

          sortedLabels = sortedEntries.map(([label]) => String(label));
          sortedFrequencies = sortedEntries.map(([, frequency]) => frequency);

          const textureNoData = sortedLabels.map((label) => (label === 'undefined' || label === 'null' || label === '' ? 'nonData' : ''));

          return [
            {
              type: 'bar' as PlotType,
              x: sortedLabels,
              y: sortedFrequencies,
              marker: {
                color: primaryColor,
                line: {
                  color: 'primaryColor',
                  width: 1,
                },
              },
              customdata: textureNoData,
            },
          ];
        } else {
          const textureNoData = xAxisData.map((label) => (label === 'undefined' || label === 'null' || label === '' ? 'nonData' : ''));

          return [
            {
              type: 'histogram' as PlotType,
              x: xAxisData,
              marker: { color: primaryColor },
              customdata: textureNoData,
            },
          ];
        }
      case 'pie':
        return [
          {
            type: 'pie' as PlotType,
            labels: xValues.map(String),
            values: xAxisData,
            marker: { colors: mainColors },
          },
        ];
      default:
        return [];
    }
  })();

  const layout: Partial<Plotly.Layout> = {
    xaxis: {
      title: showAxis ? (xAxisLabel ? xAxisLabel : '') : '',
      showgrid: false,
      visible: showAxis,
      ...(typeof xAxisData[0] === 'string' || (plotType === 'histogram' && sortedLabels.length > 0)
        ? { type: 'category', categoryarray: sortedLabels, categoryorder: 'array' }
        : {}),
      showline: true,
      zeroline: false,
    },
    yaxis: {
      title: showAxis ? (yAxisLabel ? yAxisLabel : '') : '',
      showgrid: false,
      visible: showAxis,
      showline: true,
      zeroline: false,
    },
    font: {
      family: 'Inter, sans-serif',
      size: 12,
      color: '#374151',
    },
    hoverlabel: {
      bgcolor: 'rgba(255, 255, 255, 0.8)',
      bordercolor: 'rgba(0, 0, 0, 0.2)',
      font: {
        family: 'Inter, sans-serif',
        size: 14,
        color: '#374151',
      },
    },
  };
  console.log('run plot');
  return { plotData, layout };
};

export const CustomChartPlotly: React.FC<CustomChartPlotlyProps> = ({
  xAxisData,
  plotType,
  title,
  showAxis,
  yAxisData,
  xAxisLabel,
  yAxisLabel,
}) => {
  const internalRef = useRef<HTMLDivElement>(null);
  const [divSize, setDivSize] = useState({ width: 0, height: 0 });
  const [hoveredPoint, setHoveredPoint] = useState<{ left: number; top: number; xValue: number; yValue: number } | null>(null);

  useEffect(() => {
    const handleResize = () => {
      if (internalRef.current) {
        const { width, height } = internalRef.current.getBoundingClientRect();
        setDivSize({ width, height });
      }
    };

    handleResize();
    window.addEventListener('resize', handleResize);
    if (internalRef.current) {
      new ResizeObserver(handleResize).observe(internalRef.current);
    }

    return () => {
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  const handleHover = (event: any) => {
    const { points } = event;
    if (points.length) {
      const point = points[0];
      const plotRect = internalRef.current?.getBoundingClientRect();

      if (plotRect) {
        const xIndex = point.xaxis.d2p(point.x);
        const yIndex = point.yaxis.d2p(point.y);

        setHoveredPoint({
          left: plotRect.left + xIndex,
          top: plotRect.top + yIndex,
          xValue: point.x,
          yValue: point.y,
        });
      }
    }
  };

  const handleUnhover = () => {
    setHoveredPoint(null);
  };

  const { plotData, layout } = preparePlotData(xAxisData, plotType, yAxisData, xAxisLabel, yAxisLabel, showAxis);

  useEffect(() => {
    const svg = document.querySelector('svg');
    if (svg) {
      // Create or find the `defs` section
      let defs = svg.querySelector('defs');
      if (!defs) {
        defs = document.createElementNS('http://www.w3.org/2000/svg', 'defs');
        svg.insertBefore(defs, svg.firstChild);
      }

      // Check if the pattern already exists
      let pattern = defs.querySelector('#diagonalHatch');
      if (!pattern) {
        // Create the diagonal hatch pattern
        pattern = document.createElementNS('http://www.w3.org/2000/svg', 'pattern');
        pattern.setAttribute('id', 'diagonalHatch');
        pattern.setAttribute('width', '6');
        pattern.setAttribute('height', '6');
        pattern.setAttribute('patternTransform', 'rotate(45)');
        pattern.setAttribute('patternUnits', 'userSpaceOnUse');

        const rect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
        rect.setAttribute('width', '2');
        rect.setAttribute('height', '6');
        rect.setAttribute('fill', '#cccccc');

        pattern.appendChild(rect);
        defs.appendChild(pattern);
      }

      //const bars = select('.points').selectAll('path').nodes();
      const bars = document.querySelectorAll('.points path');
      console.log(bars);
      if (plotType === 'histogram') {
        bars.forEach((bar, index) => {
          const customData = (plotData[0] as any).customdata[index];
          console.log(select(bar), customData, customData == 'nonData');
          select(bar).style('fill', 'rgb(250, 0, 0)');

          if (customData == 'nonData') {
            //select(bar).style('fill', 'url(#diagonalHatch)');
          }
          console.log(bar);
        });
      }
    }
  }, [plotData]);

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden relative" ref={internalRef}>
      <Plot
        data={plotData}
        config={{
          responsive: true,
          scrollZoom: false,
          displayModeBar: false,
          displaylogo: false,
        }}
        layout={{
          ...layout,
          width: divSize.width,
          height: divSize.height,
          title: title,
          dragmode: false,
        }}
        onHover={handleHover}
        onUnhover={handleUnhover}
      />
      {/* 

      {hoveredPoint && (
        <div>
          <Tooltip open={true} showArrow={true}>
            <TooltipTrigger />
            <TooltipContent
              style={{
                position: 'absolute',
                left: hoveredPoint.left,
                top: hoveredPoint.top,
                transform: 'translate(-50%, 0%)',
                transform: 'translate(-50%, 0%)',
              }}
            >
              <div>
                <strong>{hoveredPoint.xValue}</strong>: {hoveredPoint.yValue}
              </div>
            </TooltipContent>
          </Tooltip>
        </div>
      )}
        */}
    </div>
  );
};
